package net.illuimago.cjs.alphatab;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileEditor.FileEditorManagerEvent;
import com.intellij.openapi.fileEditor.FileEditorManagerListener;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.util.JDOMExternalizable;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.DefaultJDOMExternalizer;
import com.intellij.openapi.util.WriteExternalException;

import javax.swing.*;
import java.awt.*;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NonNls;
import org.jdom.Element;

/*
 * Copyright (c) 2000-2007 C.J. Schmitt
 * schmclug, 20.03.2007 08:08:03
 *
 * This program is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation;
 * either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * See http://www.gnu.org/licenses.
 */

public final class AlphaTab implements ProjectComponent, FileEditorManagerListener, Configurable, JDOMExternalizable
{
    private final Project p;
    private AlphaTabConfigurationForm confForm;
    private final AlphaTabReorder reorder;

    public boolean truncate = false;
    public boolean highlight = true;
    public String numChars = "15";
    public boolean caseSensitive = false;

    private static final String ALPHA_TAB = "AlphaTab";

    public AlphaTab(final Project project)
    {
        this.p = project;
        this.reorder = new AlphaTabReorder(this);
     }

    public final void initComponent()
    {
        final FileEditorManager fem = FileEditorManager.getInstance(this.p);
        fem.addFileEditorManagerListener(this);
    }

    public final void disposeComponent()
    {
        final FileEditorManager fem = FileEditorManager.getInstance(this.p);
        fem.removeFileEditorManagerListener(this);
    }

    @NotNull
    public final String getComponentName()
    {
        return ALPHA_TAB;
    }

    public final void projectOpened()
    {
    }

    public final void projectClosed()
    {
    }

    //-- FileEditorManagerListener --

    public final void fileOpened(final FileEditorManager source, final VirtualFile file)
    {
        this.reorder.reorder(this.p);
    }

    public final void fileClosed(final FileEditorManager source, final VirtualFile file)
    {
        this.reorder.reorder(this.p);
    }

    public final void selectionChanged(final FileEditorManagerEvent event)
    {
        this.reorder.reorder(this.p);
    }

    public boolean isTruncateTitle()
    {
        return this.truncate;
    }

    public void setTruncateTitle(final boolean truncateTitle)
    {
        this.truncate = truncateTitle;
    }

    public String getNumOfCharsInTitle()
    {
        return this.numChars;
    }

    public int getNumOfCharsInTitleAsInt()
    {
        try
        {
            return Integer.valueOf(this.numChars);
        }
        catch (NumberFormatException e)
        {
            return 0;
        }
    }

    public void setNumOfCharsInTitle(final String numOfCharsInTitle)
    {
        this.numChars = numOfCharsInTitle;
    }

    public boolean isHighlightTitle()
    {
        return this.highlight;
    }

    public void setHighlightTitle(final boolean highlightTitle)
    {
        this.highlight = highlightTitle;
    }

    public boolean isCaseSensitive()
    {
        return caseSensitive;
    }

    public void setCaseSensitive(boolean caseSensitive)
    {
        this.caseSensitive = caseSensitive;
    }

    public void readExternal(final Element element) throws InvalidDataException
    {
        DefaultJDOMExternalizer.readExternal(this, element);
    }

    public void writeExternal(final Element element) throws WriteExternalException
    {
        DefaultJDOMExternalizer.writeExternal(this, element);
    }

    @Nls
    public String getDisplayName()
    {
        return ALPHA_TAB;
    }

    public Icon getIcon()
    {
        return new ImageIcon(AlphaTab.class.getResource("icon.gif"));
    }

    @Nullable
    @NonNls
    public String getHelpTopic()
    {
        return null;
    }

    public JComponent createComponent()
    {
        if (this.confForm == null)
            this.confForm = new AlphaTabConfigurationForm();
        return this.confForm.getRootComponent();
    }

    public boolean isModified()
    {
        return this.confForm != null && this.confForm.isModified(this);
    }

    public void apply() throws ConfigurationException
    {
        if (this.confForm != null) {
            // Get data from form to component
            this.confForm.getData(this);
        }
    }

    public void reset()
    {
        if (this.confForm != null) {
            // Reset form data from component
            this.confForm.setData(this);
        }
    }

    public void disposeUIResources()
    {
        this.confForm = null;
        this.reorder.reorder(this.p);
    }

}
