/*
 * Copyright (c) 2000-2007 C.J. Schmitt
 * schmclug, 15.08.2007 08:23:19
 *
 * This program is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation;
 * either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * See http://www.gnu.org/licenses.
 */


package net.illuimago.cjs.alphatab;

import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;

import javax.swing.*;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;
import java.awt.*;

public class AlphaTabReorder
{
    private final AlphaTab alphaTabPlugin;
    private final Vector<TabPanelInfo> tabPanels = new Vector<TabPanelInfo>();
    private boolean running = false;
    private boolean firstTime = true;


    public AlphaTabReorder(final AlphaTab alphaTabPlugin)
    {
        this.alphaTabPlugin = alphaTabPlugin;
    }

    void reorder(final Project p)
    {
        final FileEditorManager fileEditorManager = FileEditorManager.getInstance(p);
        final Editor editor = fileEditorManager.getSelectedTextEditor();
        final JTabbedPane editorTabbedPane = this.getEditorTabbedPane(editor);

        if (editorTabbedPane != null)
        {
            if (this.firstTime)
            {
                editorTabbedPane.setUI(new AlignedTabbedPaneUI(this.alphaTabPlugin));
                this.firstTime = false;
            }
            if (this.isReorderAllowed(editorTabbedPane))
            {
                if (!running)
                {
                    this.running = true;

                    tabPanels.removeAllElements();
                    for (int i = 0; i < editorTabbedPane.getTabCount(); i++)
                    {
                        tabPanels.add(new TabPanelInfo(editorTabbedPane.getTitleAt(i),
                                editorTabbedPane.getIconAt(i),
                                editorTabbedPane.getComponentAt(i),
                                editorTabbedPane.getToolTipTextAt(i),
                                editorTabbedPane.getForegroundAt(i)));
                    }
                    Collections.sort(tabPanels, new Comparator<TabPanelInfo>()
                    {
                        public int compare(final TabPanelInfo tab1, final TabPanelInfo tab2)
                        {
                            String t1 = tab1.title + tab1.toolTip;
                            String t2 = tab2.title + tab2.toolTip;
                            
                            if (!AlphaTabReorder.this.alphaTabPlugin.isCaseSensitive())
                            {
                                t1 = t1.toLowerCase();
                                t2 = t2.toLowerCase();
                            }
                            return t1.compareTo(t2);
                        }
                    });
                    TabPanelInfo tabInfo;
                    for (int i1 = 0, sz = tabPanels.size(); i1 < sz; i1++)
                    {
                        tabInfo = tabPanels.get(i1);
                        editorTabbedPane.addTab(tabInfo.title, tabInfo.icon, tabInfo.component, tabInfo.toolTip);
                    }
                    for (int i11 = 0, sz = tabPanels.size(); i11 < sz; i11++)
                    {
                        tabInfo = tabPanels.get(i11);
                        editorTabbedPane.setForegroundAt(i11, tabInfo.foreground);
                    }
                    tabPanels.removeAllElements();

                    fileEditorManager.openFile(FileDocumentManager.getInstance().getFile(editor.getDocument()), true);
                }
                else
                {
                    // System.err.println("null!");
                }
                this.running = false;
            }
        }
    }

    private boolean isReorderAllowed(final JTabbedPane p)
    {
        final int tabPlacement = p.getTabPlacement();
        final int tabLayoutPolicy = p.getTabLayoutPolicy();

        return (tabPlacement == JTabbedPane.LEFT ||
                tabPlacement == JTabbedPane.RIGHT ||
                tabLayoutPolicy == JTabbedPane.SCROLL_TAB_LAYOUT);
    }

    private JTabbedPane getEditorTabbedPane(final Editor editor)
    {
        JTabbedPane tabbedPane = null;
        if (editor != null)
        {
            Component component = editor.getComponent();
            while (component != null)
            {
                if (component instanceof JTabbedPane)
                {
                    tabbedPane = (JTabbedPane) component;
                }
                component = component.getParent();
            }
        }
        return tabbedPane;
    }

    private static final class TabPanelInfo
    {
        final String title;
        final Icon icon;
        final Component component;
        final String toolTip;
        final Color foreground;

        public TabPanelInfo(final String title, final Icon icon, final Component component, final String toolTip, final Color foreground)
        {
            this.component = component;
            this.icon = icon;
            this.title = title;
            this.toolTip = toolTip;
            this.foreground = foreground;
        }
    }
}
